#region Using Directives

using System;

using ValidationFramework;

#endregion

/// <summary>
/// Summary description for Person
/// </summary>
public class Person : ValidatableBase
{

    #region Private Fields

    private string firstName;
    private string lastName;
    private int age;
    private string emailAddress;
    private CardType cardType;
    private string creditCardNumber;
    private decimal orderTotal;

    #endregion


    #region Public Constants Strings

    //Define run-time constants here so UI code has compile-time
    //checks against misspellings in data binding expressions.
    public const string FirstNameMember = "FirstName";
    public const string LastNameMember = "LastName";
    public const string AgeMember = "Age";
    public const string EmailAddressMember = "EmailAddress";
    public const string CardTypeMember = "CardType";
    public const string CreditCardNumberMember = "CreditCardNumber";
    public const string OrderTotalMember = "OrderTotal";

    #endregion


    #region Public Properties

    [RequiredValidator("First name is required.")]
    [LengthValidator(50, "First name must be less than 50 characters.")]
    public string FirstName
    {
        get
        {
            return firstName;
        }
        set
        {
            firstName = value;
        }
    }

    [RequiredValidator("Last name is required.")]
    [LengthValidator(2, 50, "Last name must be between 2-50 characters.")]
    public string LastName
    {
        get
        {
            return lastName;
        }
        set
        {
            lastName = value;
        }
    }

    [RequiredValidator("Age is required.")]
    [CompareValidator(18, ValidationCompareOperator.GreaterThanEqual, "Age must be at least 18 years old.")]
    public int Age
    {
        get
        {
            return age;
        }
        set
        {
            age = value;
        }
    }

    [RequiredValidator("Email is required.")]
    [RegexValidator(@"\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*", "Invalid email format.")]
    [LengthValidator(50, "Email must be less than 50 characters.")]
    public string EmailAddress
    {
        get
        {
            return emailAddress;
        }
        set
        {
            emailAddress = value;
        }
    }

    [CustomValidator("Credit card is invalid.", "CheckValidCreditCard")]
    public CardType CardType
    {
        get
        {
            return cardType;
        }
        set
        {
            cardType = value;
        }
    }

    [RequiredValidator("Credit card number is required.")]
    [LengthValidator(16, "Credit card cannot exceed 16 characters.")]
    [CustomValidator("Credit card is invalid.", "CheckValidCreditCard")]
    public string CreditCardNumber
    {
        get
        {
            return creditCardNumber;
        }
        set
        {
            creditCardNumber = value;
        }
    }

    [RequiredValidator("Order total is required.")]
    [RangeValidator(10, 1000, "Order must be between $10-$1000.")]
    public decimal OrderTotal
    {
        get
        {
            return orderTotal;
        }
        set
        {
            orderTotal = value;
        }
    }

    #endregion


    #region Custom validation methods

    private void CheckValidCreditCard(object sender, CustomValidationEventArgs e)
    {
        CustomValidationEventArgs args = IsCreditCardValid(this.cardType, this.creditCardNumber);
        e.IsValid = args.IsValid;
        e.ErrorMessage = args.ErrorMessage;
    }

    /// <summary>
    /// Provide a public static validation method so Web UI can re-use validation logic
    /// </summary>
    /// <param name="card"></param>
    /// <param name="cardNumber"></param>
    /// <returns></returns>
    public static CustomValidationEventArgs IsCreditCardValid(CardType card, string cardNumber)
    {
        CustomValidationEventArgs e = new CustomValidationEventArgs();

        if (cardNumber == null)
        {
            e.IsValid = true;
            return e;
        }

        switch (card)
        {
            case CardType.Invalid:
                e.IsValid = false;
                e.ErrorMessage = "Card type not specified.";
                break;
            case CardType.Visa:
            case CardType.MasterCard:
                e.IsValid = (cardNumber.Length == 16);
                e.ErrorMessage = "VISA/Mastercard gotta have 16 characters";
                break;
            case CardType.Amex:
                e.IsValid = (cardNumber.Length == 15);
                e.ErrorMessage = "AMEX gotta have 15 characters.";
                break;
            default:
                throw new ArgumentException("Invalid CardType");
        }
        return e;
    }

    #endregion

}

public enum CardType
{
    Invalid,
    Visa,
    MasterCard,
    Amex
}
