#region Using Directives

using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Text;

using ValidationFramework;

#endregion

namespace ValidationDemo.Win
{
    public class Person : NotifyValidatableBase
    {

        #region Private Fields

        private string firstName;
        private string lastName;
        private int age;
        private string emailAddress;
        private CardType cardType;
        private string creditCardNumber;
        private decimal orderTotal;

        //eventually use next 2 to demo Compare validator
        //private string cardFirstName;
        //private string cardLastName;

        #endregion


        #region Public Constants Strings

        //Define run-time constants here so UI code has compile-time
        //checks against misspellings in data binding expressions.
        public const string FirstNameMember = "FirstName";
        public const string LastNameMember = "LastName";
        public const string AgeMember = "Age";
        public const string EmailAddressMember = "EmailAddress";
        public const string CardTypeMember = "CardType";
        public const string CreditCardNumberMember = "CreditCardNumber";
        public const string OrderTotalMember = "OrderTotal";

        #endregion


        #region Public Properties

        [RequiredValidator("First name is required.")]
        [LengthValidator(50, "First name must be less than 50 characters.")]
        public string FirstName
        {
            get
            {
                return firstName;
            }
            set
            {
                if (firstName != value)
                {
                    firstName = value;
                    this.NotifyAndValidate(FirstNameMember);
                }
            }
        }

        [RequiredValidator("Last name is required.")]
        [LengthValidator(2, 50, "Last name must be between 2-50 characters.")]
        public string LastName
        {
            get
            {
                return lastName;
            }
            set
            {
                if (lastName != value)
                {
                    lastName = value;
                    this.NotifyAndValidate(LastNameMember);
                }
            }
        }

        [RequiredValidator("Age is required.")]
        [CompareValidator(18, ValidationCompareOperator.GreaterThanEqual, "Age must be at least 18 years old.")]
        public int Age
        {
            get
            {
                return age;
            }
            set
            {
                if (age != value)
                {
                    age = value;
                    this.NotifyAndValidate(AgeMember);
                }
            }
        }

        [RequiredValidator("Email is required.")]
        [RegexValidator(@"\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*", "Invalid email format.")]
        [LengthValidator(50, "Email must be less than 50 characters.")]
        public string EmailAddress
        {
            get
            {
                return emailAddress;
            }
            set
            {
                if (emailAddress != value)
                {
                    emailAddress = value;
                    this.NotifyAndValidate(EmailAddressMember);
                }
            }
        }

        [CustomValidator("Credit card is invalid.", "CheckValidCreditCard")]
        public CardType CardType
        {
            get
            {
                return cardType;
            }
            set
            {
                if (cardType != value)
                {
                    cardType = value;
                    this.NotifyAndValidate(CardTypeMember);
                }
            }
        }

        [RequiredValidator("Credit card number is required.")]
        [LengthValidator(16, "Credit card cannot exceed 16 characters.")]
        [CustomValidator("Credit card is invalid.", "CheckValidCreditCard")]
        public string CreditCardNumber
        {
            get
            {
                return creditCardNumber;
            }
            set
            {
                if (creditCardNumber != value)
                {
                    creditCardNumber = value;
                    this.NotifyAndValidate(CreditCardNumberMember);
                }
            }
        }

        [RequiredValidator("Order total is required.")]
        [RangeValidator(10, 1000, "Order must be between $10-$1000.")]
        public decimal OrderTotal
        {
            get
            {
                return orderTotal;
            }
            set
            {
                if (orderTotal != value)
                {
                    orderTotal = value;
                    this.NotifyAndValidate(OrderTotalMember);
                }
            }
        }

        #endregion


        #region Custom validation methods

        [SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        private void CheckValidCreditCard(object sender, CustomValidationEventArgs e)
        {
            if (this.creditCardNumber == null)
            {
                e.IsValid = true;
                return;
            }

            switch (this.cardType)
            {
                case CardType.Invalid:
                    e.IsValid = false;
                    e.ErrorMessage = "Card type not specified.";
                    break;
                case CardType.Visa:
                case CardType.MasterCard:
                    e.IsValid = (this.creditCardNumber.Length == 16);
                    e.ErrorMessage = "VISA/Mastercard gotta have 16 characters";
                    break;
                case CardType.Amex:
                    e.IsValid = (this.creditCardNumber.Length == 15);
                    e.ErrorMessage = "AMEX gotta have 15 characters.";
                    break;
                default:
                    throw new ArgumentException("Invalid CardType");
            }
        }

        #endregion

    }

    public enum CardType
    {
        Invalid,
        Visa,
        MasterCard,
        Amex
    }
}


