#region Using Directives

using System;
using System.Collections.Generic;
using System.ComponentModel;

#endregion

namespace ValidationFramework
{
	/// <summary>
	/// Provides base class that developer can (optionally) inherit from to provide
    /// INotifyPropertyChanged and IDataErrorInfo functionality for all sub-classes.
    /// This ideal for windows forms applications to get immediate validation feedback
    /// on databound controls.
    /// 
    /// An example sub-class property would look like this:
    /// <example>
    /// [RequiredValidator("Last name is required.")]
    /// public string LastName
    /// {
    ///     get { return this.lastName; }
    ///     set
    ///     {
    ///         if (lastName != value)
    ///         {
    ///             this.lastName = value;
    ///             this.NotifyAndValidate(LastNameMember);
    ///         }
    ///     }
    /// }
    /// </example>
	/// Author: Steve Michelotti
	/// </summary>
	public abstract class NotifyValidatableBase : ValidatableBase, IDataErrorInfo, INotifyPropertyChanged
	{

		#region Properties

		public override bool IsValid
		{
			get
			{
                //Override this property to JUST return bool value because all
                //validations have already been done for each property
                return this.Validator.IsValid;
			}
		}

		#endregion


        #region Protected Methods

        /// <summary>
        /// Perform validation and INotifyPropertyChanged functionality for 
        /// specified property.
        /// </summary>
        /// <param name="propertyName"></param>
        protected void NotifyAndValidate(string propertyName)
        {
            this.Validator.Validate(propertyName);
            this.NotifyPropertyChanged(propertyName);
        }

        /// <summary>
        /// Performs INotifyPropertyChanged functionality for specified property.
        /// </summary>
        /// <param name="propertyName"></param>
        protected void NotifyPropertyChanged(string propertyName)
        {
            this.OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
        }

        /// <summary>
        /// Perform Validation for specified property.
        /// </summary>
        /// <param name="propertyName"></param>
        protected void ValidateProperty(string propertyName)
        {
            this.Validator.Validate(propertyName);
        }

        #endregion


        #region IDataErrorInfo Members

        string IDataErrorInfo.this[string columnName]
        {
	        get
	        {
                return this.Validator.GetErrorMessages(columnName);
	        }
        }

		protected string this[string columnName]
		{
			get
			{
                return this.Validator.GetErrorMessages(columnName);
			}
		}

		string IDataErrorInfo.Error
		{
			get
			{
                return this.GetAllValidationMessages(Environment.NewLine);
			}
		}

		protected string Error
		{
			get
			{
                return this.GetAllValidationMessages(Environment.NewLine);
			}
		}

		#endregion


        #region INotifyPropertyChanged Members

        private event PropertyChangedEventHandler propertyChanged;

        public event PropertyChangedEventHandler PropertyChanged
        {
            add
            {
                propertyChanged += value;
            }
            remove
            {
                propertyChanged -= value;
            }
        }

        protected virtual void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            if (propertyChanged != null)
            {
                propertyChanged(this, e);
            }
        }

        #endregion

    }//end class
}//end namespace
