#region Using Directives

using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Text;

#endregion

namespace ValidationFramework
{
    /// <summary>
    /// Provides base class that developer can (optionally) inherit from to provide
    /// minimal base functionality to sub-classes.
    /// Author: Steve Michelotti
    /// </summary>
    public abstract class ValidatableBase : IValid
    {

        #region Private Fields

        private ValidationManager validator;

        #endregion


        #region Properties

        protected ValidationManager Validator
        {
            get
            {
                if (this.validator == null)
                {
                    //Lazy initialize so that ValidationManager only instantiated when needed
                    this.validator = new ValidationManager(this);
                }
                return this.validator;
            }
        }

        /// <summary>
        /// Property can be overridden when a parent object needs to validate children.
        /// </summary>
        public virtual bool IsValid
        {
            get
            {
                return (this as IValid).IsValid;
            }
        }

        bool IValid.IsValid
        {
            get
            {
                //Base behavior is to perform validations and return boolean value.
                //Sub-class can override this if, for example, they are validating on the fly.
                this.Validator.Validate();
                return this.Validator.IsValid;
            }
        }

        public virtual IEnumerable<string> ValidationMessages
        {
            get
            {
                return this.Validator.ViolationMessages;
            }
        }

        #endregion


        #region Methods

        public string GetAllValidationMessages(string separator)
        {
            return string.Join(separator, new List<string>(this.ValidationMessages).ToArray());
        }

        /// <summary>
        /// Determines whether IValid objects are valid while ensuring that the validation
        /// routines are executed for all items. For NotifyValidatableBase, this check
        /// is not necessary since all valdations have already been performed.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="items"></param>
        /// <returns></returns>
        [SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        protected bool ValidateItems(params IValid[] items)
        {
            if (items == null)
            {
                throw new ArgumentNullException("items", "Items cannot be null.");
            }

            bool invalidFound = false;
            foreach (IValid item in items)
            {
                if (!item.IsValid)
                {
                    invalidFound = true;
                }
            }
            return !invalidFound;
        }

        #endregion

    }
}
