#region Using Directives

using System;

#endregion

namespace ValidationFramework
{
	/// <summary>
	/// Performs a Required Field validation.
	/// Author: Steve Michelotti
	/// </summary>
	[AttributeUsage(AttributeTargets.Property)]
	public sealed class RequiredValidatorAttribute : ValidatorAttribute
	{

		#region Private Fields

		private string initValue;

		#endregion


		#region Public Properties

		public string InitialValue
		{
			get
			{
				return this.initValue;
			}
		}

		#endregion


		#region Constructors

		public RequiredValidatorAttribute(string initialValue, string errorMessage) : base(errorMessage)
		{
			this.initValue = initialValue;
		}
		
		public RequiredValidatorAttribute(string errorMessage) : this(null, errorMessage)
		{
		}

		#endregion
		
		
		#region Protected Overrides

        protected internal override bool EvaluateIsValid()
		{
            /** In order to leverage the generic method, have to use if/then
             * statement here to satisfy compiler requirements for "struct" 
             * generic constraint.  We cannot simply cast to ValueType because
             * nullable types are not allowed with the "struct" constraint.
             **/
            if (PropertyToValidate.Type == typeof(int))
            {
                int value = Convert.ToInt32(this.PropertyToValidate.Value);
                int initialValue = (this.initValue == null ? default(int) : Convert.ToInt32(this.initValue));
                return Validation.IsRequiredValid(value, initialValue);
            }
            else if (PropertyToValidate.Type == typeof(byte))
            {
                byte value = Convert.ToByte(this.PropertyToValidate.Value);
                byte initialValue = (this.initValue == null ? default(byte) : Convert.ToByte(this.initValue));
                return Validation.IsRequiredValid(value, initialValue);
            }
            else if (PropertyToValidate.Type == typeof(short))
            {
                short value = Convert.ToInt16(this.PropertyToValidate.Value);
                short initialValue = (this.initValue == null ? default(short) : Convert.ToInt16(this.initValue));
                return Validation.IsRequiredValid(value, initialValue);
            }
            else if (PropertyToValidate.Type == typeof(DateTime))
            {
                DateTime value = Convert.ToDateTime(this.PropertyToValidate.Value);
                DateTime initialValue = (this.initValue == null ? default(DateTime) : Convert.ToDateTime(this.initValue));
                return Validation.IsRequiredValid(value, initialValue);
            }
            else if (PropertyToValidate.Type == typeof(decimal))
            {
                decimal value = Convert.ToDecimal(this.PropertyToValidate.Value);
                decimal initialValue = (this.initValue == null ? default(decimal) : Convert.ToDecimal(this.initValue));
                return Validation.IsRequiredValid(value, initialValue);
            }
            else if (PropertyToValidate.Type == typeof(float))
            {
                float value = Convert.ToSingle(this.PropertyToValidate.Value);
                float initialValue = (this.initValue == null ? default(float) : Convert.ToSingle(this.initValue));
                return Validation.IsRequiredValid(value, initialValue);
            }
            else if (PropertyToValidate.Type == typeof(double))
            {
                double value = Convert.ToDouble(this.PropertyToValidate.Value);
                double initialValue = (this.initValue == null ? default(double) : Convert.ToDouble(this.initValue));
                return Validation.IsRequiredValid(value, initialValue);
            }
            else if (PropertyToValidate.Type == typeof(Guid))
            {
                Guid value = new Guid(this.PropertyToValidate.Value.ToString());
                Guid initialValue = (this.initValue == null ? default(Guid) : new Guid(initValue));
                return Validation.IsRequiredValid(value, initialValue);
            }
            else if (PropertyToValidate.Type == typeof(string))
            {
                string value = (this.PropertyToValidate.Value == null ? null : this.PropertyToValidate.Value.ToString());
                //Since String can be null or empty, need to do an explicit check
                if (this.initValue != null)
                {
                    return Validation.IsRequiredValid(value, initValue);
                }
                else
                {
                    return Validation.IsRequiredValid(value);
                }
            }
            else
            {
                throw new ArgumentException("Invalid data type to use for Required validator.");
            }
		}

		#endregion

	}//end class
}//end namespace
