#region Using Directives

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Reflection;

//Create a generic alias to make the code more readable
using ValidatorList = System.Collections.ObjectModel.ReadOnlyCollection<ValidationFramework.ValidatorAttribute>;

#endregion

namespace ValidationFramework
{
	/// <summary>
	/// This is the primary class that will be utilized by the consumer.
    /// This class responsible for exposing a simple public API to the 
    /// consumer while handling the internals of invoking all 
    /// validators properly.
	/// Author: Steve Michelotti
	/// </summary>
	public class ValidationManager
	{

		#region Private Fields
        
        private Dictionary<string, ValidatorAttribute> errorsDictionary = new Dictionary<string, ValidatorAttribute>();
		private object target;
		private Type targetType;

        private Dictionary<string, ValidatorList> propertyCache;

		#endregion


		#region Public Properties

		public bool IsValid
		{ 
			get
			{
				return (this.errorsDictionary.Count == 0);
			}
		}

		public ICollection<ValidatorAttribute> Violations
		{
			get
			{
                return errorsDictionary.Values;
			}
		}

        public IEnumerable<string> ViolationMessages
        {
            get
            {
                foreach (ValidatorAttribute validator in this.errorsDictionary.Values)
                {
                    yield return validator.ErrorMessage;
                }
            }
        }

		#endregion


		#region Constructors

		public ValidationManager(object targetClass)
		{
			if (targetClass == null)
			{
				throw new ArgumentNullException(ExceptionStrings.TargetParam, ExceptionStrings.TargetParamError);
			}

			this.target = targetClass;
			this.targetType = this.target.GetType();

            this.InitializePropertyCache();
		}

		#endregion
		
		
		#region Public Methods

        /// <summary>
        /// Validates all properties.
        /// </summary>
		public void Validate()
		{
            foreach (ValidatorList list in this.propertyCache.Values)
            {
                this.CheckValidProperty(list);
            }
		}

        /// <summary>
        /// Validates only the specified property.
        /// </summary>
        /// <param name="propertyName">Property to validate</param>
		public void Validate(string propertyName)
		{
            if (string.IsNullOrEmpty(propertyName))
            {
                throw new ArgumentNullException(ExceptionStrings.PropertyName, ExceptionStrings.PropertyNameError);
            }

            this.CheckValidProperty(propertyCache[propertyName]);
		}

        /// <summary>
        /// Returns a string of all error messages for a given property
        /// </summary>
        /// <param name="property">Property to retrieve error message for</param>
        /// <returns></returns>
		public string GetErrorMessages(string property)
		{
            List<string> errors = new List<string>();
			
			foreach (ValidatorAttribute validator in this.errorsDictionary.Values)
			{
				if (validator.PropertyName == property)
				{
					errors.Add(validator.ErrorMessage);
				}
			}

			string[] errorsArr = new string[errors.Count];
			errors.CopyTo(errorsArr, 0);
			return string.Join(Environment.NewLine, errorsArr);
		}

		public override string ToString()
		{
            return this.ToString(Environment.NewLine);
		}

        public string ToString(string separator)
        {
            List<string> errors = new List<string>(this.ViolationMessages);
            return string.Join(separator, errors.ToArray());
        }

		#endregion


		#region Private Methods

        /// <summary>
        /// Performs initialization so that we call Type.GetProperties()
        /// and PropertyInfo.GetCustomAttributes() ONLY in this method and
        /// then cache the results for later use.  Since the reflection
        /// methods are more expensive, the goal is to execute them only 1
        /// time (in this method) and cache the results so we do not have
        /// to invoke these for each validation.
        /// </summary>
        private void InitializePropertyCache()
        {
            this.propertyCache = new Dictionary<string, ValidatorList>();

            PropertyInfo[] properties = this.targetType.GetProperties(BindingFlags.Public | BindingFlags.Instance);
            foreach (PropertyInfo prop in properties)
            {
                ValidatorAttribute[] attributes = (ValidatorAttribute[])prop.GetCustomAttributes(typeof(ValidatorAttribute), true);
                foreach (ValidatorAttribute attr in attributes)
                {
                    //Make sure each attribute is "aware" of the property it's validating
                    attr.PropertyToValidate = new PropertyInfoExtended(prop, target);
                }
                
                if (attributes.Length != 0)
                {
                    propertyCache.Add(prop.Name, Array.AsReadOnly(attributes));
                }
            }
        }

        /// <summary>
        /// Performs validation for a specific property.
        /// </summary>
        /// <param name="validationSet">Info for property to validate</param>
        private void CheckValidProperty(ReadOnlyCollection<ValidatorAttribute> validatorList)
        {
            foreach (ValidatorAttribute attr in validatorList)
            {
                attr.Validate();

                if (!attr.IsValid)
                {
                    this.errorsDictionary[attr.UniqueKey] = attr;
                }
                else
                {
                    if (this.errorsDictionary.ContainsKey(attr.UniqueKey))
                        this.errorsDictionary.Remove(attr.UniqueKey);
                }
            }
        }

		private static class ExceptionStrings
		{
			public const string TargetParam = "targetClass";
			public const string TargetParamError = "The targetClass object cannot be null.";
            public const string PropertyName = "propertyName";
            public const string PropertyNameError = "The propertyName is invalid.";
        }

        #endregion

    }//end class
}//end namespace
