#region Using Directives

using System;
using System.Collections.Generic;
using System.Reflection;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

#endregion

namespace ValidationFramework.Web
{
    public class ValidatorGenerator
    {

        #region Private Fields

        private List<ValidationAssociation> controlList = new List<ValidationAssociation>();

        #endregion


        #region Public Methods

        public void AddValidation(string controlId, string propertyName)
        {
            controlList.Add(new ValidationAssociation(controlId, propertyName));
        }

        public void GenerateValidators(Control containerControl, object target)
        {
            Type targetType = target.GetType();
            foreach (ValidationAssociation association in controlList)
            {
                Control control = containerControl.FindControl(association.ControlID);

                PropertyInfo prop = targetType.GetProperty(association.PropertyName);
                ValidatorAttribute[] attributes = (ValidatorAttribute[])prop.GetCustomAttributes(typeof(ValidatorAttribute), true);
                foreach (ValidatorAttribute attribute in attributes)
                {
                    attribute.PropertyToValidate = new PropertyInfoExtended(prop, target);

                    //Add web UI validator for each property
                    IValidatorCreator creator = GetValidator(control, attribute);
                    if (creator != null)
                    {
                        creator.CreateValidator();
                    }
                }
            }
        }

        #endregion


        #region Private Members

        private static IValidatorCreator GetValidator(Control control, ValidatorAttribute attr)
        {
            RequiredValidatorAttribute reqAttr = attr as RequiredValidatorAttribute;
            if (reqAttr != null)
            {
                return new RequiredValidatorCreator(reqAttr, control);
            }

            LengthValidatorAttribute lengthAttr = attr as LengthValidatorAttribute;
            if (lengthAttr != null)
            {
                return new LengthValidatorCreator(lengthAttr, control);
            }

            RangeValidatorAttribute rangeAttr = attr as RangeValidatorAttribute;
            if (rangeAttr != null)
            {
                return new RangeValidatorCreator(rangeAttr, control);
            }

            RegexValidatorAttribute regexAttr = attr as RegexValidatorAttribute;
            if (regexAttr != null)
            {
                return new RegexValidatorCreator(regexAttr, control);
            }

            CompareValidatorAttribute compareAttr = attr as CompareValidatorAttribute;
            if (compareAttr != null)
            {
                return new CompareValidatorCreator(compareAttr, control);
            }

            if (attr is CustomValidatorAttribute)
            {
                return null;
            }

            throw new ArgumentException("The attribute does not derive from ValidatorAttribute.", "attr");
        }

        #region class ValidationAssociation

        private class ValidationAssociation
        {

            #region Private Fields

            private string controlID;
            private string propertyName;

            #endregion


            #region Public Properties

            public string ControlID
            {
                get { return controlID; }
            }
            
            public string PropertyName
            {
                get { return propertyName; }
            }

            #endregion


            #region Constructors

            public ValidationAssociation(string controlId, string propName)
            {
                this.controlID = controlId;
                this.propertyName = propName;
            }

            private ValidationAssociation()
            {
            }

            #endregion

        }

        #endregion

        #endregion

    }
}
